// SPDX-FileCopyrightText: 2024 g10 Code GmbH
// SPDX-FileContributor: Carl Schwan <carl.schwan@gnupg.com>
// SPDX-License-Identifier: GPL-3.0-or-later

#include <QString>

namespace PasswordGenerator
{
QString generateRandomPassword(const QString &charset, unsigned int length);
}
